"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFullEncoding = exports.getEncodingFromPrototypeSpecifier = exports.getEncodingFromConstructor = exports.getSymbolEncoding = void 0;
const errors_1 = require("./errors");
const basic_1 = require("./encodings/basic");
const utils_1 = require("./utils");
function getSymbolEncoding(x) {
    if (typeof x !== "symbol") {
        return x;
    }
    const key = (0, utils_1.getSymbolName)(x);
    if (!key) {
        throw new errors_1.SzrError(`Failed to detect symbol name for ${String(x)}`);
    }
    return {
        key: (0, utils_1.getImplicitSymbolEncodingName)(key),
        symbol: x
    };
}
exports.getSymbolEncoding = getSymbolEncoding;
function getEncodingFromConstructor(ctor) {
    if (!ctor.prototype) {
        throw new errors_1.SzrError("Failed to detect prototype from constructor.");
    }
    return getEncodingFromPrototypeSpecifier({
        prototype: ctor.prototype
    });
}
exports.getEncodingFromConstructor = getEncodingFromConstructor;
function getEncodingFromPrototypeSpecifier(specifier) {
    var _a, _b, _c, _d;
    const encoding = {};
    if (specifier.prototype === undefined) {
        throw new errors_1.SzrError("Encoding must specify prototype.");
    }
    if (typeof specifier.prototype === "function") {
        throw new errors_1.SzrError("Prototype cannot be a function. Did you supply a constructor instead?");
    }
    const proto = (_a = specifier.prototype) !== null && _a !== void 0 ? _a : basic_1.nullPlaceholder;
    encoding.prototypes = [proto];
    const className = (0, utils_1.getClassName)(proto);
    if (!className && !specifier.key) {
        throw new errors_1.SzrError(`No key has been provided, and the prototype has no name.`);
    }
    encoding.key = (_b = specifier.key) !== null && _b !== void 0 ? _b : (0, utils_1.getImplicitClassEncodingName)(className);
    encoding.encode = (_c = specifier.encode) !== null && _c !== void 0 ? _c : (0, basic_1.getPrototypeEncoder)(proto);
    encoding.decoder = (_d = specifier.decoder) !== null && _d !== void 0 ? _d : (0, basic_1.getPrototypeDecoder)(proto);
    return encoding;
}
exports.getEncodingFromPrototypeSpecifier = getEncodingFromPrototypeSpecifier;
function getFullEncoding(specifier) {
    if (typeof specifier === "symbol" || "symbol" in specifier)
        return getSymbolEncoding(specifier);
    if (typeof specifier === "function")
        return getEncodingFromConstructor(specifier);
    if ("prototype" in specifier)
        return getEncodingFromPrototypeSpecifier(specifier);
    if (!specifier.prototypes || specifier.prototypes.length === 0) {
        throw new errors_1.SzrError("Encoding must specify prototypes.");
    }
    if (!("decoder" in specifier && "encode" in specifier)) {
        throw new errors_1.SzrError("Multi-prototype specifier must have both decoder and encode.");
    }
    if (!specifier.key) {
        throw new errors_1.SzrError("Multi-prototype specifier must provide a key.");
    }
    return specifier;
}
exports.getFullEncoding = getFullEncoding;

//# sourceMappingURL=encoding-constructors.js.map