"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.typedArrayEncodings = exports.typedArrayCtors = exports.createTypedArrayEncoding = exports.ArrayBufferEncoding = void 0;
const utils_1 = require("../utils");
const base64_js_1 = require("base64-js");
exports.ArrayBufferEncoding = {
    key: (0, utils_1.getLibraryString)("ArrayBuffer"),
    prototypes: [ArrayBuffer.prototype],
    encode(input, ctx) {
        const b64 = (0, base64_js_1.fromByteArray)(new Uint8Array(input));
        return b64;
    },
    decoder: {
        create(encodedValue, ctx) {
            const byteArray = (0, base64_js_1.toByteArray)(encodedValue);
            return byteArray.buffer;
        }
    }
};
function createTypedArrayEncoding(ctor) {
    return {
        key: (0, utils_1.getLibraryString)(ctor.name),
        prototypes: [ctor.prototype],
        encode(input, ctx) {
            return exports.ArrayBufferEncoding.encode(input.buffer, ctx);
        },
        decoder: {
            create(encodedValue, ctx) {
                const buffer = exports.ArrayBufferEncoding.decoder.create(encodedValue, ctx);
                return new ctor(buffer);
            }
        }
    };
}
exports.createTypedArrayEncoding = createTypedArrayEncoding;
exports.typedArrayCtors = [
    Uint8Array,
    Uint16Array,
    Uint32Array,
    Uint8ClampedArray,
    Int8Array,
    Int16Array,
    Int32Array,
    Float32Array,
    Float64Array,
    DataView
];
exports.typedArrayEncodings = exports.typedArrayCtors.map(createTypedArrayEncoding);

//# sourceMappingURL=binary.js.map