"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fundamentalObjectEncodings = exports.createFundamentalObjectEncoding = exports.dateEncoding = exports.regexpEncoding = void 0;
const utils_1 = require("../utils");
exports.regexpEncoding = {
    prototypes: [RegExp.prototype],
    key: (0, utils_1.getLibraryString)("RegExp"),
    encode({ source, flags }, ctx) {
        return flags ? [source, flags] : source;
    },
    decoder: {
        create(input, ctx) {
            if (typeof input === "string") {
                return new RegExp(input);
            }
            else {
                return new RegExp(input[0], input[1]);
            }
        }
    }
};
exports.dateEncoding = {
    prototypes: [Date.prototype],
    key: (0, utils_1.getLibraryString)("Date"),
    encode(input, ctx) {
        return input.getTime();
    },
    decoder: {
        create(encodedValue, ctx) {
            return new Date(encodedValue);
        }
    }
};
function createFundamentalObjectEncoding(ctor) {
    return {
        key: (0, utils_1.getLibraryString)(ctor.name),
        prototypes: [ctor.prototype],
        encode(input, ctx) {
            return input.valueOf();
        },
        decoder: {
            create(encodedValue, ctx) {
                return new ctor(encodedValue);
            }
        }
    };
}
exports.createFundamentalObjectEncoding = createFundamentalObjectEncoding;
exports.fundamentalObjectEncodings = [
    Number,
    String,
    Boolean
].map(createFundamentalObjectEncoding);

//# sourceMappingURL=scalar.js.map