"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unrecognizedSymbolKey = exports.tryDecodeScalar = exports.tryEncodeScalar = exports.noResultPlaceholder = exports.nanEncoding = exports.negZeroEncoding = exports.negInfinityEncoding = exports.infinityEncoding = exports.undefinedEncoding = void 0;
const utils_1 = require("./utils");
exports.undefinedEncoding = "-";
exports.infinityEncoding = "Infinity";
exports.negInfinityEncoding = "-Infinity";
exports.negZeroEncoding = "-0";
exports.nanEncoding = "NaN";
exports.noResultPlaceholder = "";
function tryEncodeScalar(num) {
    if (num === null || typeof num === "boolean") {
        return num;
    }
    if (num === undefined) {
        return exports.undefinedEncoding;
    }
    if (typeof num === "bigint") {
        return `B${num}`;
    }
    if (Object.is(num, -0)) {
        return exports.negZeroEncoding;
    }
    if (num === Infinity) {
        return exports.infinityEncoding;
    }
    if (num === -Infinity) {
        return exports.negInfinityEncoding;
    }
    if (Object.is(num, NaN)) {
        return exports.nanEncoding;
    }
    if (typeof num === "number") {
        return num;
    }
    return exports.noResultPlaceholder;
}
exports.tryEncodeScalar = tryEncodeScalar;
function tryDecodeScalar(candidate) {
    const t = typeof candidate;
    if (t === "boolean" || t === "number" || candidate === null)
        return candidate;
    switch (candidate) {
        case exports.infinityEncoding:
            return Infinity;
        case exports.negInfinityEncoding:
            return -Infinity;
        case exports.nanEncoding:
            return NaN;
        case exports.negZeroEncoding:
            return -0;
        case exports.undefinedEncoding:
            return undefined;
    }
    if (t === "string") {
        if (candidate.startsWith("B")) {
            return BigInt(candidate.slice(1));
        }
    }
    return exports.noResultPlaceholder;
}
exports.tryDecodeScalar = tryDecodeScalar;
exports.unrecognizedSymbolKey = (0, utils_1.getLibraryString)("symbol?");

//# sourceMappingURL=szr-representation.js.map